<div class="abc" style="float: left; margin-left: 15%; margin-right:15%; width: 70%" >
    <h2>{{ clonetypes_parameters.section_header }}    <i class="fa fa-question-circle" onClick="toggle1(this)" style="cursor:pointer;"></i> </h2>
    <div class="box1">
      <div class="description" style="display: none;">
        <p>Top 10 clonotypes by frequency. For the full table, please refer to the "*clonotypes.tsv" file produced by the pipeline.</p>
      </div>
      <table>
        <tr>
          {% for item in clonetypes_table_header %}            
            <th>{{item}}</th>  
          {% endfor %}
        </tr>
        {% for item in clonetypes_table %}
          <tr>
            {% for i in item %} 
            <td>{{ i }}</td>
            {% endfor %}
          </tr>
        {% endfor %}
      </table>

      <div class="clear" ></div>
    </div>
  </div>